/*
 * $LastChangedRevision: 1741 $
 * $LastChangedDate:: 2019-09-06 18:08:03 +0200#$
 *
 * This file is part of X2C. https://x2c.lcm.at/
 *
 * Copyright (c) 2019, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
/*
 * This file is licensed according to the BSD 3-clause license as follows:
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
 *       the names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "GlobalDefines.h"
#include "InputControl.h"

#define CURRENT_OFFSET  16384    /* 16384 = 2^15/2 */

/**
 * @brief Routine to read values from ADC.
 */
void readAnalogIn(void)
{
    /** - Bridge currents */
    I1 = (ADCBUF0 << 3) - CURRENT_OFFSET;
    I2 = (ADCBUF1 << 3) - CURRENT_OFFSET; 
    ISUM = (ADCBUF15 << 3) - CURRENT_OFFSET; 
    
    /** - Supply voltage */
    VBUS = ADCBUF18 << 3;       /* 12bit to 15bit (Q15 format) */  
    
    /** - Potentiometer */
    POT = ADCBUF19 << 3;       /* 12bit to 15bit (Q15 format) */   
}


/**
 * @brief Routine to read digital input pins.
 */
void readDigitalIn(void)
{
    /** - read push buttons */
    x2cModel.inports.bS2 = READ_S2;
    x2cModel.inports.bS3 = READ_S3;

    /** - read hall sensor signals */
    x2cModel.inports.bHA = READ_HA;
    x2cModel.inports.bHB = READ_HB;
    x2cModel.inports.bHC = READ_HC;
    
    /** - read home signal */
    x2cModel.inports.bHOME = READ_HOME;
}
